#include <stdio.h>
#define printf(...)  

/*
 * Main File including the main-routine with initializations-
 * and stack-routines.
 *
 * Author: Simon Kueppers
 * Email: simon.kueppers@web.de
 * Homepage: http://klinkerstein.m-faq.de
 * */

#include <p18cxxx.h>

#include "Hardware/Enc28j60.h"
#include "Hardware/SpiPort.h"
#include "Net/uip/uip.h"
#include "Net/uip/uip_arp.h"
#include "Net/uip/uip_split.h"
#include "Net/uip/uip_TcpAppHub.h"
#include "Net/uip/uipopt.h"

#pragma config OSC = IRCIO67 // Interner Takt
//#pragma config OSC = HS   // CPU=16 MHz
#pragma config PWRT = OFF  // Power-up Timer (the power up timer can interfere with the debugger synchronising with the debug executive)
#pragma config BOREN  = OFF  // Brown-out Reset
#pragma config WDT = OFF  //Watchdog Timer
#pragma config LVP = OFF  //Low Voltage ICSP
#pragma config XINST = ON // Extended Mode

static uint8_t g_nPrescaler = 100;

static volatile struct {
    uint8_t fPeriodic : 1;
} g_nFlags = {0};

void high_isr(void);

#pragma code high_vector=0x08
void interrupt_at_high_vector(void) {
    _asm GOTO high_isr _endasm
}

void uip_log(const rom char *data) {
	transmitUSBRomString(data);
	transmitUSBCrLf();
}

void main(void) {
    uip_ipaddr_t IpAddr;

    LATA = 0x00;
    LATB = 0x00;
    LATC = 0x00;
    LATD = 0x00;
    LATE = 0x00;
LATBbits.LATB4 = 1; // RES
LATAbits.LATA3 = 1; // CS

    TRISA = 0x00;
    TRISB = 0x00;
    TRISC = 0x00;
    TRISD = 0x00;
    TRISE = 0x00;
    CMCON = 0xff; // Turn off comparators

stdout = _H_USER;

	initUSB();
	printf("init...\r\n");

    SpiInit( );
    Enc28j60Init( );
    uip_arp_init( );
    uip_init( );
    uip_TcpAppHubInit( );
    Enc28j60SetClockPrescaler( 2 );

    // Der Preescaler so einstellen, dass ca. 10 Hertz erzeugt werden
    T0CONbits.T08BIT = 1;	// 0 - 16 bit; 1 - 8 bit
    T0CONbits.T0CS = 0;		// 0 = Internal clock (FOSC/4)
	T0CONbits.PSA = 0; 	// 0 = Timer0 prescaler is assigned. Timer0 clock input comes from prescaler output.
	T0CONbits.T0PS = 5;		// Prescaler 1:2

    INTCONbits.TMR0IF = 0; // Clear Timer0 overflow flag
    INTCONbits.TMR0IE = 1; // Enable Timer0 overflow interrupt

    T0CONbits.TMR0ON = 1; // 1 - Timer enabled

	// Dieses Board
    uip_ipaddr( IpAddr, 192, 168, 0, 2 );
    uip_sethostaddr( IpAddr );
    
	// Router
	uip_ipaddr( IpAddr, 192, 168, 0, 1 );
    uip_setdraddr( IpAddr );

	// NetMask    
	uip_ipaddr( IpAddr, 255, 255, 255, 0 );
    uip_setnetmask( IpAddr );

    RCONbits.IPEN = 1; // Enable priority levels on interrupts
    INTCONbits.GIEH = 1; // Enables all high-priority interrupts
    INTCONbits.GIEL = 0; // Disables all low-priority peripheral interrupts

	LATAbits.LATA0 = 0;

    while (1) {
        uip_len = Enc28j60Receive( uip_buf );

        if (uip_len > 0) {
			//LATAbits.LATA0 = ~LATAbits.LATA0;

            if (((struct uip_eth_hdr *) &uip_buf[0])->type == htons( UIP_ETHTYPE_IP )) {
                //uip_arp_ipin();
                uip_input( );

                if (uip_len > 0) {
                    uip_arp_out( );
                    uip_split_output( );

                }

            } else if (((struct uip_eth_hdr *) &uip_buf[0])->type == htons( UIP_ETHTYPE_ARP )) {
                uip_arp_arpin( );
                if (uip_len > 0) {
                    uip_split_output( );
                }
            }
        }

        if (g_nFlags.fPeriodic) {
            int i = UIP_CONNS;
            INTCONbits.GIEH = 0; // Enables all high-priority interrupts
            g_nFlags.fPeriodic = 0;
            INTCONbits.GIEH = 1; // Enables all high-priority interrupts

            while (i) {
                i--;
                uip_periodic( i );
                if (uip_len > 0) {
                    uip_arp_out( );
                    uip_split_output( );
                }
            }

            g_nPrescaler--;

            if (g_nPrescaler == 0) {
                //Every 10 seconds
                uip_arp_timer( );
                g_nPrescaler = 100;
				uip_log("uip_arp_timer");
            }
        }
    }
}

#pragma interrupt high_isr
void high_isr(void) {
    g_nFlags.fPeriodic = 1;

    INTCONbits.TMR0IF = 0;
}
