#include <p18cxxx.h>
//#include "hardware.h"
#include "inttypes.h"

typedef unsigned char uint8_t;

#define initUSBData()   USB_RD = 1; USB_WR = 0; 
#define dataPortRead()	TRISD = 0xff; Nop();
#define dataPortWrite() TRISD = 0x00; Nop();

#define USB_RD		LATEbits.LATE0
#define USB_WR		LATEbits.LATE1
#define USB_DATA_LATCH	LATD		// auch LCD
#define USB_DATA_PORT	PORTD		// auch LCD

#define initUSBPort()	TRISA |= 0x10; TRISB |= 0x03;
#define USB_PWREN	PORTAbits.RA4
#define USB_RXF		PORTBbits.RB0
#define USB_TXE		PORTBbits.RB1

void initUSB(void) {
    initUSBPort( );
    initUSBData( );
}

/*
 * Wartet, bis USB verbunden ist.
 * "Goes low after the device is configured by USB, then high during USB suspend."
 */
void _WtUSB() {
    while (USB_PWREN == 1) {
        ClrWdt( ); // clear watchdogtimer
    };
}

/**
 * Wartet auf den Empfang des nchsten Bytes
 * "When high, do not write data into the FIFO. 
 * When low, data can be written into the FIFO by strobing WR high, then low."
 */
void _waitRxUSB(void) {
    while (USB_RXF == 1) {
        ClrWdt( ); // clear watchdogtimer
    }
}

/**
 * Wartet auf Sendebereitschaft
 * "When high, do not read data from the FIFO. 
 * When low, there is data available in the FIFO 
 * which can be read by strobing RD# low, then high again."
 */
void _waitTxUSB(void) {
    while (USB_TXE == 1) {
        ClrWdt( ); // clear watchdogtimer
    }
}

/**
 * Liest ein Byte vom USB-Port
 */
uint8_t _receiveUSB(void) {
    uint8_t byte;

    _waitRxUSB( );

    dataPortRead( ); // switch Port to Input
    USB_RD = 0; // activate RD#
    Nop( );
    byte = USB_DATA_PORT;
    USB_RD = 1; // deactivate RD#
    return byte;
}

/**
 * Schreibt ein Byte zum USB-Port
 */
void transmitUSB(uint8_t byte) {
    _waitTxUSB( );

    dataPortWrite( ); // switch Port to Output
    USB_DATA_LATCH = byte;
    USB_WR = 1;
    Nop( );
    USB_WR = 0;
}

/**
 * Sendet ein Digit (0..F) ber USB als ASCII
 */
void transmitUSBDigit(uint8_t byte) {
    byte = (byte & 0x0F) + '0';
    if (byte > '9') {
        byte += 7;
    }
    transmitUSB( byte );
}

/**
 * Das Byte wird als HEX mit anschlieendem Leerzeichen zum USB-Port gesendet.
 */
void transmitUSBHex(uint8_t byte) {
    transmitUSBDigit( (byte & 0xF0) >> 4 );
    transmitUSBDigit( byte & 0x0F );
    transmitUSB( ' ' );
}

void transmitUSBCrLf(void) {
    transmitUSB( 0x0d );
    transmitUSB( 0x0a );
}

void transmitUSBRawSeq(void) {
    transmitUSB( 0xaa );
    transmitUSB( 0x55 );
}

/**
 *
 */
void transmitUSBRomString(const rom char *data) {
    while (*data) {
        transmitUSB( *data++ );
    }
}

// fr printf
#include <stdio.h>

int _user_putc (char c) {
	transmitUSB(c);
	return c;
}
